# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.drivers.alinity import (
    DriverAlinity,
)
from driverng.astm import (
    AstmMessageV2,
)


class DriverAlinityMh(DriverAlinity):
    SPEC = {
        "code": "alinity_mh",
        "label": "Alinity m/h HL7 Biomoléculaire"
    }

    OPTIONS = [
        AstmMessageV2.Option("alinity_sender_code", label="Code émetteur", default="LISApp"),
        AstmMessageV2.Option("alinity_sender_label", label="Libellé émetteur", default="MainFacility"),
        AstmMessageV2.Option("alinity_receiver_code", label="Code récepteur", default="AlinityApp"),
        AstmMessageV2.Option("alinity_receiver_label", label="Libellé récepteur", default="LabFacility"),
        AstmMessageV2.BooleanOption(
            "alinity_dilution_en_alarme",
            label="Traiter le facteur du dilution comme alarme du résultat"
        ),
        AstmMessageV2.Option(
            "alinity_code_exception",
            label="Code exception à mettre dans le résultat si flag X dans OBX.11",
            default="EXCEPTION"
        )
    ]

    def __init__(
            self,
            alinity_sender_code="LISApp",
            alinity_sender_label="MainFacility",
            alinity_receiver_code="AlinityApp",
            alinity_receiver_label="LabFacility",
            alinity_dilution_en_alarme=False,
            alinity_code_exception="EXCEPTION",
            **kwargs
    ):
        self.sender_code = alinity_sender_code
        self.sender_label = alinity_sender_label
        self.receiver_code = alinity_receiver_code
        self.receiver_label = alinity_receiver_label
        self.dilution_en_alarme = alinity_dilution_en_alarme
        self.code_exception = alinity_code_exception
        super(DriverAlinityMh, self).__init__(**kwargs)

    def encode_worklist(self, worklist, context):
        message = self.create(
            sending_application=self.sender_code,
            sending_facility=self.sender_label,
            receiving_application=self.receiver_code,
            receiving_facility=self.receiver_label,
            message_date=context["now"],
            control_id=context.get(
                "control_id", int(time.time() * 1000))
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority", "rerun"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "CA", worklist.ELSE: "NW"},
        ):
            for sample, action, priority, rerun in patient.samples:
                seq = 1
                for order in sample["orders"]:
                    seg_spm = message.specimens.add(
                        specimen_type=sample.specimen,
                        role_id="P",
                        role_label="Patient",
                        role_coding_system="HL70369",
                        collection_date=sample.collection_date
                    )
                    seg_sac = seg_spm.infos_samples.add(
                        identifier=sample.sample_id
                    )
                    if not rerun and order.dilution:
                        seg_sac.dilution = order.dilution
                    seg_sac.common_order.add(
                        order_control=action,
                        date_transaction=sample.request_date
                    )
                    if priority == "R":
                        priority_label = "Routine"
                    elif priority == "S":
                        priority_label = "STAT"
                    seg_tq1 = seg_sac.sample_priority.add(
                        priority_code=priority,
                        priority_label=priority_label,
                        priority_coding="HL70485"
                    )
                    seg_obr = seg_tq1.orders.add(
                        placer_order_number=context.get("placer_order_number", int(time.time() * 1000 + seq)),
                        test_identifier=order.code,
                        test_text=order.code,
                    )
                    for physician in sample.physicians:
                        seg_obr.ordering_provider = physician.code
                    if rerun and order.dilution:
                        seg_obr.infos_tests.add(
                            universal_service_identifier=order.code,
                            test_label=order.code,
                            test_coding_name="99ABT",
                            auto_dilution_type=order.dilution,
                            dilution_coding="99ABT",
                        )
                    seq += 1
        if len(message.specimens):
            return message
        elif context["query"]:
            for sample_id in [seg_qpd.query_name for seg_qpd in context["query"].queries]:
                seg_spm = message.specimens.add(
                    role_id="U",
                    role_label="Unknow",
                    role_coding_system="HL70369"
                )
                seg_sac = seg_spm.infos_samples.add(identifier=sample_id)
                seg_sac.common_order.add(
                    order_control="DC",
                    date_transaction=context["now"]
                )
            return message
